package com.safeway.epe.offersdataproducer.resource;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.safeway.epe.offersdataproducer.appconfiguration.AppConfigs;
import com.safeway.epe.offersdataproducer.domodel.*;
import com.safeway.epe.offersdataproducer.service.OffersDataProducerService;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.kafka.common.TopicPartition;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.skyscreamer.jsonassert.JSONAssert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.kafka.support.SendResult;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.SettableListenableFuture;

import java.util.concurrent.ExecutionException;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT) // for restTemplate
@ActiveProfiles("test")
public class OffersDataProducerResourceTest {
    @InjectMocks
    private OffersDataProducerResource offersDataProducerResource;
    @Mock
    private OffersDataProducerService offersDataProducerService;
    @Autowired
    private TestRestTemplate testRestTemplate;
    @Test
    public void publishOfferEventTest() throws JsonProcessingException, ExecutionException, InterruptedException {
        OffersDataDO offersDataDO = buildOffersDataDO();
        ObjectMapper mapper = new ObjectMapper();
        String excpected = mapper.writeValueAsString(offersDataDO);
        SettableListenableFuture future = new SettableListenableFuture();
        ProducerRecord<Integer, String> producerRecord = new ProducerRecord(AppConfigs.TOPIC_NAME, null, offersDataDO);
        RecordMetadata recordMetadata = new RecordMetadata(new TopicPartition(AppConfigs.TOPIC_NAME, 0),
                8, 1, 342, System.currentTimeMillis(), -1, 581);
        SendResult<Integer, String> sendResult = new SendResult<Integer, String>(producerRecord, recordMetadata);

        future.set(sendResult);
        Mockito.when(offersDataProducerService.publishOfferEvent(Mockito.any(OffersDataDO.class)))
                .thenReturn(future);
        ResponseEntity<ListenableFuture<SendResult<String, String>>> result = offersDataProducerResource.publishOfferEvent(offersDataDO);
        SendResult<String, String> sendResult1 = result.getBody().get();
        assert sendResult1.getRecordMetadata().partition() == 0;
        String actual = mapper.writeValueAsString(result.getBody().get().getProducerRecord().value());
        JSONAssert.assertEquals(excpected ,actual ,false);
    }
    private OffersDataDO buildOffersDataDO() {
        DiscountTierDO discountTierDO = DiscountTierDO
                .builder().amount(6.99f).upTo("3")
                .itemLimit(5).weightLimit(2).build();
        PointsDO pointsDO = PointsDO.builder().generalPoints(5).loyaltyPoints(250).build();
        DiscountDO discountDO = DiscountDO
                .builder().discountTierDO(Lists.newArrayList(discountTierDO))
                .allowNegative(false).flexNegative(false).build();
        BenefitDO benefitDO = BenefitDO
                .builder().discountDO(Lists.newArrayList(discountDO))
                .pointsDO(Lists.newArrayList(pointsDO)).benefitValueType("A")
                .build();
        IdDO idDO = IdDO.builder().manufacturerId("MMM").offerId(135918444).build();
        InfoDO infoDO = InfoDO
                .builder().offerId(135918444).idDO(idDO).offerProgramCode("SC")
                .terminalsDO(Lists.newArrayList("Groceryworks (Safeway.com)",
                        "Smart Cart (QVS Virtual Terminals)",
                        "Bakery")).build();
        RulesDO rulesDO = RulesDO.builder().offerId(135918444).benefitDO(benefitDO).endDate("2021-02-27T06:59:58.000+00:00")
                .startDate("2019-07-27T07:00:00.000+00:00").build();
        OffersDO offersDO = OffersDO.builder().infoDO(infoDO).rulesDO(rulesDO).build();
        OffersDataDO offersDataDO = OffersDataDO.builder().offerId(135918444).offersDO(Lists.newArrayList(offersDO)).build();

        return offersDataDO;
    }

}
